<?php 
class WCGFI_Cart
{
	var $already_processed = array();
	var $all_products_present_in_restriction_filter = array();
	public function __construct()
	{
		add_action( 'woocommerce_cart_calculate_fees', array(&$this, 'compute_free_items') );
	}
	public function compute_free_items()
	{
		global $woocommerce, $wcgfi_product_model, $wcgfi_promotion_model,$wcgfi_wpml_helper;
		$at_least_one_promotion_has_been_applied = false;
		$active_promotions = $wcgfi_promotion_model->get_active_promotions();
		$this->already_processed = array();
		$applied_coupons = $woocommerce->cart->get_coupons();
		
		foreach($active_promotions as $promotion)
		{
			//Coupon check 
			// $promotion['apply_promotion_only_for_selected_coupon'] ;
			if($promotion['do_not_apply_promotion_if_any_coupon_has_been_applied_to_cart'] && count($applied_coupons) > 0)
				continue;
			
			$skip_promotion = false;
			//wcgfi_var_dump($skip_promotion);
			if(count($promotion['apply_promotion_only_for_selected_coupon']) > 0)
			{
				$skip_promotion = true;
				foreach((array)$applied_coupons as $coupon_code => $coupon_data)
				{
					//wcgfi_var_dump("coupon id: ".$coupon_data->get_id());
					//wcgfi_var_dump($promotion['apply_promotion_only_for_selected_coupon']);
					//wcgfi_var_dump(in_array($coupon_data->get_id(),  $promotion['apply_promotion_only_for_selected_coupon']));
					if(in_array($coupon_data->get_id(),  $promotion['apply_promotion_only_for_selected_coupon']))
						$skip_promotion = false;
				}
			}
			if($skip_promotion)
				continue;
			/* wcgfi_var_dump("**********");			
			wcgfi_var_dump($skip_promotion);  */
			
			// I
			$matching_promotion_data = $promotion['promotion_application_strategy'] == 'cart_total' ? $this->total_is_matching_the_promotion($promotion) : $this->is_cart_containing_the_selected_items($promotion);
			
			if(!$matching_promotion_data['is_match'])
				continue;
			
			if($at_least_one_promotion_has_been_applied && $promotion['do_not_apply_if_any_promotion_has_been_applied'])
				continue;
			
			// II
			$cheapest_item_data = $this->find_cart_cheapest_or_most_expensive_product($promotion, $matching_promotion_data);
			
			if(!isset($cheapest_item_data['item'])) 
				continue;
			
			if(isset($this->already_processed[$cheapest_item_data['item']["product_id"]."-".$cheapest_item_data['item']["variation_id"]]))
				continue; 
			
			//III
			$at_least_one_promotion_has_been_applied = true;
			$this->already_processed[$cheapest_item_data['item']["product_id"]."-".$cheapest_item_data['item']["variation_id"]] = true;
			$woocommerce->cart->add_fee( $wcgfi_wpml_helper->translate_single_string($promotion['unique_id'], $promotion['promotion_name']),  $cheapest_item_data['discount']*-1, $cheapest_item_data['item']['data']-> get_tax_status() == 'taxable'); 
		}					
	}
	// I 
	private function total_is_matching_the_promotion($promotion)
	{
		global $woocommerce;
		$cart = $woocommerce->cart;
		$items = $woocommerce->cart->get_cart();
		$total = $cart->subtotal_ex_tax;
		$result = array('is_match' => false, 'times_to_apply_promotion' => 0);
		
		//Detect "frequency" promotion. APPLIED EVEN IF ONLY ONE ITEM IN CART
		if($promotion['promotion_application_method'] == 'every_n_item_or_amount')
		{
			if($promotion['cart_total_n_value'] != 0 && $total/$promotion['cart_total_n_value'] > 1)
			{
				//wcgfi_var_dump("every_n_item_or_amount applied");
				$result['is_match'] = true;
				$result['times_to_apply_promotion'] = floor($total/$promotion['cart_total_n_value']);
				return $result;
			}
			
			return $result;
		}
		
		//Cart has to contain more than 1 item
		if(!$this->is_cart_containing_minium_product_quantity($items))
			return $result; //false

		if( $total > $promotion['cart_total_min'] && ($promotion['cart_total_max'] == 'none' || $total < $promotion['cart_total_max']))
		{
			$result['is_match'] = true;
			return $result;
		}
		
		return $result; //false
	}
	private function is_cart_containing_the_selected_items($promotion)
	{
		global $woocommerce;
		//$cart = $woocommerce->cart;
		$items = $woocommerce->cart->get_cart();
		$this->all_products_present_in_restriction_filter = array();
		$all_restriction_are_satisfied = false;
		$result = array('is_match' => false, 'times_to_apply_promotion' => 0);
		$promotion_application_time_per_item_set = array();
		
		//Cart has to contain more than 1 item. In case "apply promotion every N items", promotions is APPLIED EVEN IF ONLY ONE ITEM IN CART
		if($promotion['promotion_application_method'] != 'every_n_item_or_amount' && !$this->is_cart_containing_minium_product_quantity($items))
			return false; //false
		
		//Min and Max are the sum of the selected products
		foreach($promotion['cart_products_restriction'] as $cart_restriction)
		{
			$sum_of_selected_product_quantities = $distinct_items_count = 0;
			$this->all_products_present_in_restriction_filter = !$cart_restriction['which_products_categories_to_consider'] ? array_merge($this->all_products_present_in_restriction_filter, $cart_restriction['product_ids']) : $this->all_products_present_in_restriction_filter;
			$cart_restriction['quantity_max'] == 0 ? $cart_restriction['quantity_max'] = 'none' : $cart_restriction['quantity_max'];
			$min_quantity = -1;
			
			if($cart_restriction['quantity_policy'] == 'cumulative')
			{
				foreach($items as $key => $item)
				{
					if( $cart_restriction['which_products_categories_to_consider'] || in_array($item['product_id'], $cart_restriction['product_ids']) || in_array($item['variation_id'], $cart_restriction['product_ids']))
					{
						$sum_of_selected_product_quantities += $item['quantity'];
						$distinct_items_count++;
					}
				}
				$min_quantity = $sum_of_selected_product_quantities;
				
				//wcgfi_var_dump($sum_of_selected_product_quantities);
				$cart_restriction['quantity_max'] == 0 ? $cart_restriction['quantity_max'] = 'none' : $cart_restriction['quantity_max'];
				if($promotion['promotion_application_method'] == 'every_n_item_or_amount')
				{
					//Detect "frequency" promotion.
					$cart_restriction['quantity_min'] = $cart_restriction['quantity_n'];
					$cart_restriction['quantity_max'] = 'none';
				}
				if($sum_of_selected_product_quantities >= $cart_restriction['quantity_min'] && 
					($cart_restriction['quantity_max'] == 'none' || $sum_of_selected_product_quantities <= $cart_restriction['quantity_max'])
				  )
					$all_restriction_are_satisfied = true;
				
			}
			else //distinct
			{
				
				//init 
				$all_restriction_are_satisfied = !empty($items) ? true : false;
				foreach($items as $key => $item)
				{
					$current_product_belongs_to_selected_items = $cart_restriction['which_products_categories_to_consider']  || in_array($item['product_id'], $cart_restriction['product_ids']) || in_array($item['variation_id'], $cart_restriction['product_ids']) ;
					$distinct_items_count = $current_product_belongs_to_selected_items ? $distinct_items_count+1 : $distinct_items_count;
					//wcgfi_var_dump($item['product_id']." ".$item['quantity']." ".$cart_restriction['quantity_min']." ".$cart_restriction['quantity_max']);
					//wcgfi_var_dump($current_product_belongs_to_selected_items);
					if($current_product_belongs_to_selected_items)
					{
						//Detect "frequency" promotion.
						if($promotion['promotion_application_method'] == 'every_n_item_or_amount')
						{
							$cart_restriction['quantity_min'] = $cart_restriction['quantity_n'];
							$cart_restriction['quantity_max'] = 'none';
						}
						
						if( $all_restriction_are_satisfied &&  
							$item['quantity'] >= $cart_restriction['quantity_min'] && 
							($cart_restriction['quantity_max'] == 'none' || $item['quantity'] <= $cart_restriction['quantity_max'])
						  )
						{
							$all_restriction_are_satisfied = true;
							$min_quantity = $min_quantity == -1 || $item['quantity'] < $min_quantity ? $item['quantity'] : $min_quantity ;
							//break;
						}
						else
						{
							$all_restriction_are_satisfied = false;
						}
					}
				}
				/* if(!$all_restriction_are_satisfied )
					break; */
			}
			$promotion_application_time_per_item_set[] = floor($min_quantity/$cart_restriction['quantity_n']);
			
			if(!$all_restriction_are_satisfied )
					break;
				
			//distinc item check
			if($cart_restriction['consider_distinct_number_of_items'] == 'yes')
			{
				if( ($cart_restriction['distinct_min_item_quantity'] == 0  || $distinct_items_count >= $cart_restriction['distinct_min_item_quantity']) &&
					($cart_restriction['distinct_max_item_quantity'] == 0  || $distinct_items_count <= $cart_restriction['distinct_max_item_quantity']) )
				   {
					   //something to do?
				   }
				else 
					$all_restriction_are_satisfied = false;
			}
		}
		//return $all_restriction_are_satisfied; 
		$result['is_match'] = $all_restriction_are_satisfied;
		if($promotion['promotion_application_method'] == 'every_n_item_or_amount')
		{
			$min_application_promotion_time = -1;
			foreach($promotion_application_time_per_item_set as $item_set_time)
				$min_application_promotion_time = $min_application_promotion_time == -1 || $item_set_time < $min_application_promotion_time ? $item_set_time : $min_application_promotion_time ;
				
			$result['times_to_apply_promotion'] = $min_application_promotion_time;
		}
		return $result; 
	}
	//
	
	// II
	public function find_cart_cheapest_or_most_expensive_product($promotion, $matching_promotion_data)
	{	
		global $woocommerce;
		$cart = $woocommerce->cart;
		$items = $woocommerce->cart->get_cart();
		$result = array('item' => null, 'discount' => 0);
		//wcgfi_var_dump($matching_promotion_data);
		
		//Cart has to contain more than 1 item
		if(!$this->is_cart_containing_minium_product_quantity($items))
			return $result; 
		
		if($promotion['items_to_get_strategy'] == 'cheapest_items' || $promotion['items_to_get_strategy'] == 'expensive_items')
		{
			//$cheapest_item = null;
			foreach($items as $key => $item)
			{
				//wcgfi_var_dump($item["line_total"]/$item["quantity"]);
				if(	  $result['item'] == null || 																																		  //First run
					( $promotion['items_to_get_strategy'] == 'cheapest_items' && $item["line_total"]/$item["quantity"] <= $result['item']["line_total"]/$result['item']["quantity"] ) ||  //Cheapest 
					( $promotion['items_to_get_strategy'] == 'expensive_items' && $item["line_total"]/$item["quantity"] >= $result['item']["line_total"]/$result['item']["quantity"] )    //Most expensive
					
					)
				{
					
					//Exclude product in case it among the ids used as restriction
					if($promotion['exclude_selected_items'] == 'no' || 
					  (!in_array($item['product_id'], $this->all_products_present_in_restriction_filter) && !in_array($item['variation_id'], $this->all_products_present_in_restriction_filter) )
					)
					{
						//$result['item'] = null;
						$result['item'] = $item;
					}
					
				}
			}
				
			if(isset($result['item']))
			{
				$item = $result['item'];
				//$result['item'] = $cheapest_item;
				
				//In case of "Give item free every N items"
				if($matching_promotion_data['times_to_apply_promotion'] > 0)
				{
					$promotion['cheapest_item_quantity'] = $promotion['cheapest_item_quantity'] != "none" ?  $matching_promotion_data['times_to_apply_promotion'] * $promotion['cheapest_item_quantity'] : $matching_promotion_data['times_to_apply_promotion'];
				}
				$quantity = $promotion['cheapest_item_quantity'] != "none" && $item["quantity"] >= $promotion['cheapest_item_quantity'] ? $promotion['cheapest_item_quantity'] : $item["quantity"];
				$result['discount'] = (($item["line_total"]/$item["quantity"]) * ($promotion['cheapest_item_discount']/100)) * $quantity;
				
			}
			
		}
		else if($promotion['items_to_get_strategy'] == 'specific_items' || $promotion['items_to_get_strategy'] == 'expensive_specific_items')
		{
			//wcgfi_var_dump($promotion['items_to_get']);
			foreach($items as $key => $item)
			{
				//wcgfi_var_dump("Processing: ".$item['data']->post->post_title.", ID: ".$item['product_id'].", price: ".$item["line_total"]/$item["quantity"]);
				foreach((array)$promotion['items_to_get'] as $specific_items)
				{
					//wcgfi_var_dump($specific_items['product_ids']);
					//wcgfi_var_dump(in_array($item['product_id'], $specific_items['product_ids']));
					if( (in_array($item['product_id'], $specific_items['product_ids']) || in_array($item['variation_id'], $specific_items['product_ids']) ) /*&&
						($specific_items['quantity_max'] == "" || $specific_items['quantity_max'] <= $item["quantity"])*/
					  )
						{
							if(	 $result['item'] == null || 																																				//First run
						    	($promotion['items_to_get_strategy'] == 'specific_items' && $item["line_total"]/$item["quantity"] <= $result['item']["line_total"]/$result['item']["quantity"]) || 			//Cheapest 
						    	($promotion['items_to_get_strategy'] == 'expensive_specific_items' && $item["line_total"]/$item["quantity"] >= $result['item']["line_total"]/$result['item']["quantity"])	//Most expensive
							  )
							{
								//wcgfi_var_dump("Found: ".$item['data']->post->post_title.", price: ".$item["line_total"]/$item["quantity"]);
								$result['item'] = $item;
								
								//In case of "Give item free every N items"
								if($matching_promotion_data['times_to_apply_promotion'] > 0)
								{
									$specific_items['quantity_max'] = $specific_items['quantity_max'] != "none" ?  $matching_promotion_data['times_to_apply_promotion'] * $specific_items['quantity_max'] : $matching_promotion_data['times_to_apply_promotion'];
								}
								$quantity = $specific_items['quantity_max'] != "none" && $item["quantity"] >= $specific_items['quantity_max'] ? $specific_items['quantity_max'] : $item["quantity"];
								$result['discount'] = (($item["line_total"]/$item["quantity"]) * ($specific_items['cheapest_item_discount']/100)) * $quantity;
								
							}
						}
				}
			}
		}
		
		return $result;
	}
	//
	private function is_cart_containing_minium_product_quantity($items) 
	{
		//wcgfi_var_dump(count($items) == 1 && array_values($items)[0]['quantity'] < 2);
		if(count($items) == 1 && array_values($items)[0]['quantity'] < 2)
			return false; 
		
		return true;
	}
}
?>